
#include "sed.h"

// die Prprozessor-Variable TEST ermglich die Verwendung
// der Klasse Engine in einer Test-MFC-Applikation

#ifndef TEST
class Engine : public CCmdTarget
{
	DECLARE_DYNCREATE(Engine)
	Engine();           // protected constructor used by dynamic creation
#else
class Engine
{
public:
	Engine(void) { Reset(); }
#endif

// Attributes
public:
   void Reset(void);

   int findnr, findnr_tmp;
	void find(void);
   CString findstr, findregexp;

   // Start *** globale sed-Variablen *** Start
   int no_default_output; /* If set, don't write out the line unless explictly told to */
   int input_line_number; /* Current input line # */
   int last_input_file;   /* Are we on the last input file? */
   int input_EOF;         /* Have we hit EOF on the last input file?  This is used to decide if we have hit the '$' address yet. */
   int quit_cmd;          /* non-zero if a quit command has been executed. */
   int replaced;          /* Have we done any replacements lately?  This is used by the 't' command. */
   int program_depth;     /* How many '{'s are we executing at the moment */
   struct vector *the_program; /* The complete compiled SED program that we are going to run */
   struct sed_label *jumps;  /* information about labels and jumps-to-labels.  This is used to do the required backpatching after we have compiled all the scripts. */
   struct sed_label *labels;
   struct line line;          /* The 'current' input line. */
   struct line hold;          /* An input line that's been stored by later use by the program */

   struct line append;        /* A 'line' to append to the current line when it comes time to write it out */
   /* When we're reading a script command from a string, 'prog_start' and
      'prog_end' point to the beginning and end of the string.  This
      would allow us to compile script strings that contain nulls, except
      that script strings are only read from the command line, which is
      null-terminated */
   unsigned char *prog_start;
   unsigned char *prog_end;

   unsigned char *prog_cur; /* When we're reading a script command from a string, 'prog_cur' points to the current character in the string */
   char *prog_name; /* This is the name of the current script file. It is used for error messages. */

   /* This is the current script file.  If it is zero, we are reading
      from a string stored in 'prog_start' instead.  If both 'prog_file'
      and 'prog_start' are zero, we're in trouble! */
   FILE *prog_file;

   int prog_line; /* this is the number of the current script line that we're compiling.  It is used to give out useful and informative error messages. */

   FILE *input_file; /* This is the file pointer that we're currently reading data from.  It may be stdin */
   char *input_puffer;
   int output_art;
   char *output_cstr;
   int output_count;

   int bad_input; /* If this variable is non-zero at exit, one or more of the input files couldn't be opened. */

   /* 'an empty regular expression is equivalent to the last regular
      expression read' so we have to keep track of the last regex used.
      Here's where we store a pointer to it (it is only malloc()'d once) */
   struct re_pattern_buffer *last_regex;
   re_registers regs; 
   FilePtrs file_ptrs[NUM_FPS];
   char *last_regex_string;
   // Ende *** globale sed-Variablen *** Ende

   // Start *** globale sed-Funktionen *** Start
   void init(void);
   void scan_regexp(char *regexp, char *expr, char cmd, int pos);
   int myeof(void);
   int mygetc(void);
   int mytesteof(int c);
   void myungetc(int ch);
   void close_files (void);
   void compile_string (char *str);
   void compile_file (char *str);
   vector *compile_program (struct vector * vector, int);
   int compile_address (struct addr * addr);
   void compile_regex (void);
   int inchar (void);
   void savchar (int ch);
   void buffer_regex  (int slash);
   sed_label *setup_jump (struct sed_label * list, struct sed_cmd * cmd, struct vector * vec);
   FILE *compile_filename (int readit);
   void ausgabe(char *ptr,int len);
   void read_string (char *name, int flagcnt=0, int flagperline=TRUE);
   void execute_program (struct vector * vec);
   int match_address (struct addr * addr);
   int read_pattern_space (int flagperline=TRUE);
   void append_pattern_space (void);
   // Ende *** globale sed-Funktionen *** Ende
   long regexpcpy(char *tmp1, char *regexp);
   CString txt[10]; 

// Operations
public:

#ifndef TEST
   // Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(Engine)
	public:
	virtual void OnFinalRelease();
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~Engine();

	// Generated message map functions
	//{{AFX_MSG(Engine)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
	DECLARE_OLECREATE(Engine)
#endif

	// Generated OLE dispatch map functions
	//{{AFX_DISPATCH(Engine)
	short m_replacePosition;
	BOOL m_replacePerLine;
	BOOL m_noMatch;
	long m_line;
	long m_position;
	long m_globalPosition;
	BOOL m_error;
	afx_msg BSTR Replace(LPCTSTR in, LPCTSTR regexp, LPCTSTR expr);
	afx_msg short Match(LPCTSTR in, LPCTSTR regexp);
	afx_msg void FindFirst(LPCTSTR in, LPCTSTR regexp);
	afx_msg void FindNext();
	afx_msg long InStr(long position, LPCTSTR in, LPCTSTR regexp);
	afx_msg BSTR GetText(short index);
	afx_msg void SetText(short index, LPCTSTR lpszNewValue);
	//}}AFX_DISPATCH
#ifndef TEST
	DECLARE_DISPATCH_MAP()
	DECLARE_INTERFACE_MAP()
#endif
};
